#-----------------------------------------------------------------------------
#
# Copyright 2006, Vendmr
# Based on CutList.rb, Copyright 2005, CptanPanic
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided something the above
# copyright notice appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#-----------------------------------------------------------------------------
#
# Name        : CutListAndMaterials.rb
#
# Type        : Tool
#
# Description : Makes a cutlist based on all selected components in model.
#
# Menu Item   : Plugins -> Generate CutList
# Context-Menu: None
#
# Author      : Vendmr
#
# Usage       : Call script using Plugins menu, and file name *-CutList.cvs is created in the model's 
#               folder.  It is readable by text editors, and cutlist programs like CutList Plus.
#               This script uses bounding boxes to ascertain the dimensions of parts within the user
#               selection.  Only components and components within components are recorded.  Enure that
#               the component axes are adjusted to the component to give the smallest bounding box to
#               get accurate sizes.  The top level component/s shoul;d be aligned to the axes of the
#               model.  Volumes are given in board foot.
#               Any component whose name starts with Part: will not be treated as wood for the cutlist
#               but as a hardware part and it will be counted.
#
# Date        : January 2007
#
# Version     :	1.0			Initial Release.
#             : 2.0         Added recognition of hardware parts
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

### do class

class Reporter

### do def

### Get the material assigned faces within a component if it is not assigned at 
### the component level
def getMaterial(component)
  bits = component.definition.entities
   for f in bits 
       if f.typename == "Face"
               materialClass = f.material
		   if(materialClass!=nil)
                  return materialClass.name
               end 
       end
   end
  
   return "Not assigned"

end ##getMaterial


### component method called recursivly
### returns true true if a component has sub components
def getSubComponents(entityList, level)
   model = Sketchup.active_model
   selection = model.selection	
   check = false
   for c in entityList
     status = selection.contains? c
     #Sub components do not appear as part of the selection so let them through
     if (status || level>1)
	
       if c.typename == "ComponentInstance"

         check = true		

         ### If Name starts with "Part:" add it to a parts array and do not dimension.
         compName = c.definition.name
         if (compName.include? "Part:")
           @@partList.add(compName)
           next   ###Skip rest of this item, ie no cutlist generated.
         end ###if

		subList = c.definition.entities
		hasSubComponents = getSubComponents(subList, level+1)
      	if (!hasSubComponents )
               name = " "+(c.definition.name) ### allows names with - + at start etc
               boundingBox = c.bounds
               dims = [ boundingBox.width.to_s,boundingBox.height.to_s,boundingBox.depth.to_s ]
               boardFeet = boundingBox.width * boundingBox.height * boundingBox.depth / 144
			   boardFeet = ((boardFeet * 1000).round).to_f/1000
               layer = " "+(c.layer.name) ### allows names with - + at start etc
               materialClass = c.material
		   if(materialClass==nil)
                  material = getMaterial(c)
               else
                  material = materialClass.name
               end 	
		   
               ### Sort dimensions longest to shortest.
		   if(boundingBox.width >= boundingBox.height && boundingBox.height >= boundingBox.depth)   		
               	@@clist = @@clist.push([name,dims[0],dims[1],dims[2],boardFeet,material])
               elsif(boundingBox.width >= boundingBox.depth && boundingBox.depth >= boundingBox.height)
               	@@clist = @@clist.push([name,dims[0],dims[2],dims[1],boardFeet,material])
               elsif(boundingBox.height >= boundingBox.width && boundingBox.width >= boundingBox.depth)
               	@@clist = @@clist.push([name,dims[1],dims[0],dims[2],boardFeet,material])
               elsif(boundingBox.height >= boundingBox.depth && boundingBox.depth >= boundingBox.width)
               	@@clist = @@clist.push([name,dims[1],dims[2],dims[0],boardFeet,material])
               elsif(boundingBox.depth >= boundingBox.width && boundingBox.width >= boundingBox.height)
               	@@clist = @@clist.push([name,dims[2],dims[0],dims[1],boardFeet,material])
               else
               	@@clist = @@clist.push([name,dims[2],dims[1],dims[0],boardFeet,material])
		   end #if
			
		end #if

         end#if 
      end#if
   end#for c 
   return check
end #getSubComponents

   @@clist = []
   ###@@partList = PartList


def Reporter::components
   @@clist = [] 
   @@partList = PartList.new()    
   model = Sketchup.active_model
### start undo...
   model.start_operation "undo"
   entities = model.entities
   mname = model.title
   selection = model.selection

### show VCB and status info...
   Sketchup::set_status_text(("Generation de la liste de coupe" ), SB_PROMPT)
   Sketchup::set_status_text(" ", SB_VCB_LABEL)
   Sketchup::set_status_text(" ", SB_VCB_VALUE)

### check model...
   mpath = Dir::pwd
   if mpath == ""
      UI.beep
      UI.messagebox("VOus devez enregistrer votre model \n avant de generer cette liste !\nAuncune liste generer.")
      return nil
   end

### setup list... ### do it instance by instance as layers might differ... TIG

 	reporter = Reporter.new	
   reporter.getSubComponents(entities,1)

   @@clist = @@clist.sort
###

### if no components exit...
   if ((not @@clist[0]) && (@@partList.parts.length == 0))
      UI.beep
      UI.messagebox("Aucun composant selectionner.\nSelectionner un composna tou crer un composant ) partir de votre selection. \nAucune liste generee.")
      return nil
   end#if
###

### write to csv file...
   namecsv = mpath + "/" + mname + "-CutList.csv"
   file = File.new(namecsv,"w")
   file.puts("Part #,Description,Length(X),Width(Y),Thickness(Z),Board Foot, Material\n") ### title ###
   i = 1
   ii = 1 ###
   ix = ""
   cx = "" ###

   materialList = []
   

   totalBF = 0	
   for c in @@clist
      i = i + 1 if c[0] != cx and cx != ""
      ii = 1 if c[0] != cx
      ix = " "+i.to_s+"-"+ii.to_s ### avoids 'date' format
      file.puts(ix+","+c[0]+","+c[1]+","+c[2]+","+c[3]+","+c[4].to_s+","+c[5])
      ### gives sub-part numbers to same named compos; last = total
      ii = ii + 1 ###
      cx = c[0] ###
	## Add the board feet
	totalBF = totalBF + c[4]
	inList = false
	for d in materialList 
	   if(d[0]==c[5])
            d[1] = d[1]+c[4]
            inList = true	
         end
	end

      if(!inList)
         materialList = materialList.push([c[5],c[4]])
      end


   end#for c

### Write out the total board feet

      file.puts("")
      file.puts("Total Board Foot"+","+totalBF .to_s)
      file.puts("")
      file.puts("Material, Board Foot")
	for d in materialList 
         file.puts(d[0]+","+d[1].to_s)
	end

### Write out the parts list

      file.puts("")
      file.puts("Part,Part Count")
	  if(@@partList.parts.length > 0)
	    for e in 0..(@@partList.parts.length-1)
          file.puts(" "+@@partList.parts[e]+","+@@partList.partCount[e].to_s)         		  
	    end  ###for
      end ###if		


   file.close
  UI.messagebox("CutList written into: \n\n" + namecsv + "  \n")
###
### commit undo...
   model.commit_operation

end#def components

end#class Reporter


### Class PartList
class PartList

### Constructor
  def initialize()
    @parts = Array.new
    @partCount = Array.new
  end  ### Constructor

   def parts
     @parts
   end ###def parts

   def partCount
     @partCount
   end ###def partCount

   ### Adds a part to the list.
   ### TODO finish this method.
   def add(inPart)
     index = @parts.index(inPart)
     if(index != nil)
       @partCount[index] = @partCount[index] + 1    
     else
       @parts.push(inPart)
       @partCount.push(1)
     end  ###if   
   end ###def add

end ###Class PartList


### do menu

if( not file_loaded?("CutList.rb") )

	cmd = UI::Command.new("Liste de decoupe")  { Reporter.components }
	cmd.tooltip = "Liste de decoupe des composants"
    cmd.menu_text = "Liste de decoupe"
    cmd.small_icon = "AdebeoIcon/listdecoupept.png"
    cmd.large_icon = "AdebeoIcon/listdecoupegd.png"
    UI.menu("Plugins").add_item cmd
    toolbar = UI::Toolbar.new "Adebeo_ensemble"
    toolbar = toolbar.add_item cmd
end#if

file_loaded("ComponentReporter.rb")

###

