#        Name : lines2walls 1.3
# Description : Creates walls starting from a selection of lines, an arc, a circle, a curve
#      Author : Didier Bur
#       Usage : select lines, an arc, a circle, a curve, select from the Plugins menu, 
#               enter wall position relative to the selection, height and width and here you go
#        Date : 11.Aug.2oo4
#       Bugs fixes of lines2wall.rb
#        Type : tool


require 'sketchup.rb'

def line_to_wall_13

model = Sketchup.active_model
model.start_operation "Lines/curves to walls"
entities = model.active_entities
ss = model.selection

if ss.empty? 
  UI.messagebox("No selection.")
  return nil
end

# sets the default wall settings
@wall_just = "Middle" if not @wall_just
$width = 10.inch if not $width
$height = 100.inch if not $height

# Dialog box
wall_just = ["Left","Middle","Right"]
enums = [wall_just.join("|")]
prompts = ["Position du mur", "Epaisseur du mur", "Hauteur du mur"]
values = [@wall_just, $width.inch, $height.inch]

results = inputbox prompts, values, enums, "Parametres des murs"

return if not results
wall_just, $width, $height = results


pis2 = 1.5707963267948965

# construct the wall profile

# We must first find a vertex that belongs to only one edge (start or end of the selection set)

i = 0
indice = 0

0.upto(ss.length - 1) do |something|
  pts = ss[i].vertices
  v1 = pts[0]
  v2 = pts[1]
  edges1 = pts[0].edges         # Array of edges that use this vertex (first endpoint of current edge)
  edges2 = pts[1].edges         # Array of edges that use this vertex (second endpoint of current edge)
  start_edge= []

  
  if( edges1.length == 1 )      # This means we have a "free" end of edge
    indice = i                   # returns indice of edge in the selection set
  break
    
  end
  if( edges2.length == 1 )      # This means we have a "free" end of edge
    indice = i                   # returns indice of edge in the selection set
  break
  end
  i = i + 1
end # of upto

# Which end of the edge is the good one ?
pts = ss[indice].vertices
if( pts[0].edges.length == 1 )
  pt1 = pts[0].position
  pt2 = pts[1].position
  vec = pt1.vector_to pt2
  else
  pt1 = pts[1].position
  pt2 = pts[0].position
  vec = pt2.vector_to pt1
end     #of if

#if( not ss.is_curve? )


if( results[0] == "Left" )
  pt1_left = pt1
  rotated_vec = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], (0 - pis2)))
  pt1_right = pt1.offset(rotated_vec, $width)
  pt2_left = pt1_left.offset([0,0,1], $height)
  pt2_right = pt1_right.offset([0,0,1], $height)
  #new_edge = ss[0].split(0.5)
  #ss.add(new_edge)
  face = Sketchup.active_model.active_entities.add_face(pt1_left, pt2_left, pt2_right, pt1_right)
end

if( results[0] == "Middle" )
  rotated_vec = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], pis2))
  pt1_left = pt1.offset(rotated_vec, $width / 2)
  rotated_vec = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], (0 - pis2)))
  pt1_right = pt1.offset(rotated_vec, $width / 2)
  pt2_left = pt1_left.offset([0,0,1], $height)
  pt2_right = pt1_right.offset([0,0,1], $height)
  face = Sketchup.active_model.active_entities.add_face(pt1_left, pt2_left, pt2_right, pt1_right)
end

if( results[0] == "Right" )
  pt1_left = pt1
  rotated_vec = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], pis2))
  pt1_right = pt1.offset(rotated_vec, $width)
  pt2_left = pt1_left.offset([0,0,1], $height)
  pt2_right = pt1_right.offset([0,0,1], $height)
  #new_edge = ss[0].split(0.5)
  #ss.add(new_edge)
  face = Sketchup.active_model.active_entities.add_face(pt1_left, pt2_left, pt2_right, pt1_right)
end

#else
  
  #rotated_vec = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], pis2))
  #pt1_left = pt1.offset(rotated_vec, $width / 2)
  #rotated_vec = vec.transform(Geom::Transformation.rotation(pt1, [0,0,1], (0 - pis2)))
  #pt1_right = pt1.offset(rotated_vec, $width / 2)
  #pt2_left = pt1_left.offset([0,0,1], $height)
  #pt2_right = pt1_right.offset([0,0,1], $height)
  #new_edge = ss[0].split(0.5)
  #ss.add(new_edge)
  #face = Sketchup.active_model.active_entities.add_face(pt1_left, pt2_left, pt2_right, pt1_right)
  #UI.messagebox("I can only draw walls like this with curves...")
  
#end

j = 0
path = []

0.upto(ss.length - 1) do |something|
  path[j] = ss[j]
  j = j + 1
end

# follow me along lines
face.reverse!.followme path

model.commit_operation
end    # of def


if( not file_loaded?("lines2walls_13.rb") )

	cmd = UI::Command.new("Extruder des lignes")  { line_to_wall_13  }
    cmd.tooltip = "Convertit les lignes en murs"
    cmd.menu_text = "Convertit les lignes en murs"
    cmd.small_icon = "AdebeoIcon/MurLignespt.png"
    cmd.large_icon = "AdebeoIcon/MurLignesgd.png"
    UI.menu("Draw").add_item cmd
    toolbar = UI::Toolbar.new "Adebeo"
    toolbar = toolbar.add_item cmd
end

#-----------------------------------------------------------------------------
file_loaded("lines2walls_13.rb")